TWILIO_ACCOUNT_SID_KEY = "TWILIO_ACCOUNT_SID"
TWILIO_AUTH_TOKEN_KEY = "TWILIO_AUTH_TOKEN"
TWILIO_PHONE_NUMBER_KEY = "TWILIO_PHONE_NUMBER"

GREEN_POINT_DRIVE_IN_CAMPSITES_ID = "-2147483314"
GREEN_POINT_WALK_IN_CAMPSITES_ID = "-2147483313"
